"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const inquirer = require("inquirer");
const Listr = require("listr");
const base_1 = require("../../base");
const github_1 = require("../../github");
const prompt = inquirer.createPromptModule();
const oldLabelMap = {
    bug: 'bug 🐞',
    enhancement: 'feature',
};
const removeLabels = ['good first issue', 'question'];
class AddOrgLabels extends base_1.default {
    constructor() {
        super(...arguments);
        this.modifyRepos = async (repos, targetLabels) => {
            const { force, concurrency, } = this.flags;
            const listrOpts = {
                renderer: (this.flags.verbose ? 'verbose' : 'default'),
                concurrent: concurrency,
                exitOnError: !force,
            };
            const tasks = repos.map(repo => {
                return {
                    title: `Modify "${repo.name}"`,
                    task: (_ctx, task) => {
                        return this.modifyRepo(repo, targetLabels, task);
                    }
                };
            });
            const listr = new Listr(tasks, listrOpts);
            return listr.run()
                .catch((err) => {
                console.error(err);
            });
        };
        this.modifyRepo = async (repo, targetLabels, task) => {
            await this.updateExistingLabels(repo, targetLabels, task);
            await this.addMisingLabels(repo, targetLabels, task);
        };
        this.updateExistingLabels = async (repoParam, targetLabels, task) => {
            const existingLabels = await this.getRepoLabels(repoParam);
            const octokit = this.octokit;
            const { owner, repo, } = repoParam;
            return Promise.all(existingLabels.map(existingLabel => {
                if (removeLabels.includes(existingLabel.name)) {
                    task.output = `🚮 Removing label "${existingLabel.name}" from repo "${repo}"`;
                    return octokit.issues.deleteLabel({ owner, repo, name: existingLabel.name })
                        .then(() => { });
                }
                const newName = oldLabelMap[existingLabel.name];
                const targetLabel = targetLabels.find(l => l.name === newName || l.name === existingLabel.name);
                if (!targetLabel) {
                    task.output = `🤷 No target label found for "${existingLabel.name}" from repo "${repo}"`;
                    this.log(`no target label found for "${existingLabel.name}"`);
                    return Promise.resolve();
                }
                const updateProps = {
                    owner,
                    repo,
                    current_name: existingLabel.name,
                    name: targetLabel.name,
                    color: targetLabel.color,
                    description: targetLabel.description,
                };
                return octokit.issues.updateLabel(updateProps)
                    .then(() => {
                    task.output = `💄 Updated label "${targetLabel.name}" on repo "${repo}"`;
                });
            }));
        };
        this.addMisingLabels = async (repoParam, targetLabels, task) => {
            const existingLabels = await this.getRepoLabels(repoParam);
            const existingLabelNames = existingLabels.map(l => l.name);
            const octokit = this.octokit;
            const { owner, repo, } = repoParam;
            return Promise.all(targetLabels.map(targetLabel => {
                if (existingLabelNames.includes(targetLabel.name)) {
                    task.output = `👀 Label "${targetLabel.name}" already exists on repo "${repo}"`;
                    return Promise.resolve();
                }
                return octokit.issues.createLabel({
                    owner,
                    repo,
                    name: targetLabel.name,
                    color: targetLabel.color,
                    description: targetLabel.description,
                })
                    .then(() => {
                    task.output = `✨ Created label "${targetLabel.name}" on repo "${repo}"`;
                });
            }));
        };
    }
    async run() {
        this.userConfig = await this.requireConfig();
        this.octokit = await github_1.getAuthenticatedOctokit(this.userConfig);
        const repos = await this.getReposToModify();
        if (!repos.length) {
            this.log('No repos to modify.');
            this.exit(0);
        }
        const targetLabels = await this.getTargetLabels();
        const result = await this.modifyRepos(repos, targetLabels);
        console.log(result);
    }
    async getTargetLabels() {
        return this.getRepoLabels({
            owner: 'RoundtableLearning',
            repo: 'Roundtable',
        });
    }
    async getRepoLabels(repo) {
        const result = await this.octokit.issues.getLabels({
            owner: repo.owner,
            repo: repo.repo,
        });
        return result.data;
    }
    async getReposToModify() {
        this.log('Listing repos...');
        const allRepos = await github_1.getRepos({
            authToken: this.userConfig.authToken,
            repoQuery: this.flags['github-query'],
            filter: this.flags.filter,
        })
            .catch(err => {
            console.log(err);
            process.exit(1);
        })
            .then(_r => (_r || []));
        if (!allRepos.length) {
            this.error('No repos returned from GitHub.');
        }
        return prompt([{
                type: 'checkbox',
                message: 'Which repos to add the labels to?',
                name: 'reposToModify',
                pageSize: 20,
                choices: allRepos.map(r => {
                    return {
                        value: r,
                        name: r.repo,
                    };
                }).sort((a, b) => a.name.localeCompare(b.name)),
            }])
            .then(({ reposToModify }) => reposToModify);
    }
}
AddOrgLabels.description = chalk_1.default `Mirrors the labels from RoundtableLearning/Roundtable to the selected repos.

Mirrors the labels from RoundtableLearning/Roundtable to the selected repos.

 - {bold {bgRed DELETES}} the default labels:
    ${removeLabels.map(l => `"${l}"`).join('\n    ')}
 - {bold {bgBlue RENAMES}} the labels:
    ${Object.entries(oldLabelMap).map(([oldVal, newVal]) => `"${chalk_1.default.bold(oldVal)}" => "${chalk_1.default.bold(newVal)}"`).join('\n    ')}
`;
AddOrgLabels.hidden = true;
AddOrgLabels.examples = [
    '$ rtolms admin:add-org-labels'
];
AddOrgLabels.flags = Object.assign({}, base_1.default.flags, { force: command_1.flags.boolean({ description: 'Do not stop on errors' }), verbose: command_1.flags.boolean({ char: 'v', description: 'Verbose' }), filter: command_1.flags.string({
        description: 'RegExp to filter names of repos',
        default: '^lms-(core|custom)',
        required: true,
    }), 'github-query': command_1.flags.string({
        description: 'query passed to the github search API to find list of repos to apply the change to',
        default: 'org:RoundtableLearning lms in:name archived:false',
        required: true,
    }) });
exports.default = AddOrgLabels;
