"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const inquirer = require("inquirer");
const Listr = require("listr");
const base_1 = require("../../base");
const github_1 = require("../../github");
const prompt = inquirer.createPromptModule();
class AddReposToTeam extends base_1.default {
    constructor() {
        super(...arguments);
        this.modifyRepos = async (repos, team) => {
            const userConfig = await this.getConfig();
            const { force, concurrency, } = this.flags;
            const listrOpts = {
                renderer: (this.flags.verbose ? 'verbose' : 'default'),
                concurrent: concurrency,
                exitOnError: !force,
            };
            const tasks = repos.map(repo => {
                return {
                    title: `Modify "${repo.name}"`,
                    task: (_ctx, task) => {
                        const modifyRepoParams = {
                            team_id: team.id,
                            owner: repo.owner,
                            repo: repo.repo,
                            permission: this.args.role,
                            authToken: userConfig.authToken,
                        };
                        return this.modifyRepo(modifyRepoParams, task);
                    }
                };
            });
            const listr = new Listr(tasks, listrOpts);
            return listr.run()
                .catch((err) => {
                console.error(err);
            });
        };
        this.modifyRepo = async (params, _task) => {
            return github_1.addRepoToTeam(params);
        };
        this.getTeam = async (params) => {
            const { teamName, } = this.args;
            const teams = await github_1.getTeams({ authToken: params.authToken });
            let team = teams.find(t => t.name === teamName);
            return Promise.resolve()
                .then(() => {
                if (team) {
                    return team;
                }
                this.log(`No team found with name "${teamName}".\n`);
                return prompt([
                    {
                        type: 'list',
                        message: 'Which team do you want to apply this change to?',
                        name: 'selectedTeam',
                        choices: teams.map(t => ({
                            name: `${t.name} (${t.description})`,
                            value: t.name,
                        })),
                    },
                ])
                    .then(({ selectedTeam }) => {
                    team = teams.find(t => t.name === selectedTeam);
                    return team;
                });
            });
            return github_1.addRepoToTeam(params);
        };
    }
    async run() {
        const userConfig = await this.requireConfig();
        const { role, } = this.args;
        const team = await this.getTeam({ authToken: userConfig.authToken });
        this.log('Getting repos...');
        const repos = await github_1.getRepos({
            authToken: userConfig.authToken,
            repoQuery: this.flags['github-query'],
            filter: this.flags.filter,
        })
            .catch(err => {
            console.log(err);
            process.exit(1);
        });
        if (repos) {
            this.log(`Assigning ${role} permissions to ${repos.length} repos...`);
            return this.modifyRepos(repos, team);
        }
    }
}
AddReposToTeam.description = 'Adds repos to the support team';
AddReposToTeam.hidden = true;
AddReposToTeam.examples = [
    '$ rtolms admin:add-repos-to-team Engineering push',
    '$ rtolms admin:add-repos-to-team Support',
    '$ rtolms admin:add-repos-to-team "Project Management" push',
];
AddReposToTeam.args = [
    {
        name: 'teamName',
        required: true,
    },
    {
        name: 'role',
        options: ['pull', 'push'],
        default: 'pull',
    },
];
AddReposToTeam.flags = Object.assign({}, base_1.default.flags, { force: command_1.flags.boolean({ description: 'Do not stop on errors' }), verbose: command_1.flags.boolean({ char: 'v', description: 'Verbose' }), filter: command_1.flags.string({
        description: 'RegExp to filter names of repos',
        default: '^lms-(core|custom)',
        hidden: true,
        required: true,
    }), 'github-query': command_1.flags.string({
        description: 'query passed to the github search API to find list of repos to apply the change to',
        default: 'org:RoundtableLearning lms in:name archived:false',
        required: true,
    }) });
exports.default = AddReposToTeam;
