"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const execa = require("execa");
const fs = require("fs-extra");
const Listr = require("listr");
const path = require("path");
const url_1 = require("url");
const base_1 = require("../base");
const flags_1 = require("../flags");
const github_1 = require("../github");
class Clone extends base_1.default {
    constructor() {
        super(...arguments);
        this.cloneRepos = async (repos) => {
            const userConfig = await this.getConfig();
            const { force, concurrency, } = this.flags;
            const listrOpts = {
                renderer: (() => {
                    if (!process.stdout.isTTY)
                        return 'verbose';
                    return this.flags.verbose ? 'verbose' : 'default';
                })(),
                concurrent: concurrency,
                exitOnError: !force,
            };
            const tasks = repos.map(repo => {
                const directory = path.join(userConfig.reposDir, repo.name);
                return {
                    title: `Clone "${repo.name}"`,
                    skip: () => {
                        return fs.pathExists(directory);
                    },
                    task: (_ctx, task) => {
                        return this.cloneRepo({
                            repo,
                            directory,
                        }, task);
                    }
                };
            });
            const listr = new Listr(tasks, listrOpts);
            return listr.run()
                .catch((err) => {
                console.error(err);
            });
        };
        this.cloneRepo = async (repoJob, task) => {
            const userConfig = await this.getConfig();
            const { repo, directory, } = repoJob;
            const { 'dry-run': dry, 'auth-type': authType, } = this.flags;
            let gitCmd;
            switch (authType) {
                case 'private-key': {
                    gitCmd = `git clone ${repo.ssh_url} ${directory}`;
                    break;
                }
                case 'token': {
                    const { username, authToken, } = userConfig;
                    const gitUrl = new url_1.URL(repo.clone_url);
                    gitUrl.username = username;
                    gitUrl.password = authToken;
                    gitCmd = `git clone ${gitUrl.toString()} ${directory}`;
                    break;
                }
                default:
                //pass
            }
            if (dry) {
                task.output = chalk_1.default `{gray [dry run] would execute:} ${gitCmd}`;
                return Promise.resolve();
            }
            task.output = chalk_1.default `{gray clone command:} ${gitCmd}`;
            return execa.shell(gitCmd);
        };
    }
    async run() {
        const userConfig = await this.requireConfig();
        this.log('Getting repos...');
        const repos = await github_1.getRepos({
            authToken: userConfig.authToken,
            repoQuery: this.flags['github-query'],
            filter: this.flags.filter,
        })
            .catch(err => {
            console.log(err);
            process.exit(1);
        });
        if (repos) {
            return this.cloneRepos(repos);
        }
    }
}
Clone.description = 'Updates local working copies';
Clone.examples = [
    '$ rtolms clone',
];
Clone.flags = Object.assign({}, base_1.default.flags, { 'dry-run': command_1.flags.boolean({
        char: 'd',
        description: 'Don\'t actually perform clone',
    }), 'auth-type': command_1.flags.string({
        char: 'a',
        description: 'Which authentication type to use',
        options: ['private-key', 'token'],
        default: 'token',
    }), concurrency: flags_1.concurrency({
        char: 'c',
        default: 4,
        required: true,
    }), force: command_1.flags.boolean({ description: 'Do not stop on errors' }), verbose: command_1.flags.boolean({ char: 'v', description: 'Verbose' }), filter: command_1.flags.string({
        description: 'RegExp to filter names of repos',
        default: '^lms-(core|custom)',
        hidden: true,
        required: true,
    }), 'github-query': command_1.flags.string({
        description: 'query passed to the github search API to find list of repos to clone',
        default: 'org:RoundtableLearning lms in:name archived:false',
        hidden: true,
        required: true,
    }) });
exports.default = Clone;
