"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const fs = require("fs-extra");
const globby = require("globby");
const inquirer = require("inquirer");
const Listr = require("listr");
const os = require("os");
const path = require("path");
const base_1 = require("../base");
const utils_1 = require("../utils"); //tslint:disable-line no-unused no-unused-variable
const unlink_1 = require("./unlink");
const prompt = inquirer.createPromptModule();
class Link extends base_1.default {
    constructor() {
        super(...arguments);
        this.getLinkTarget = async () => {
            const { target: targetFlag } = this.flags;
            if (targetFlag) {
                return fs.stat(targetFlag)
                    .then(stats => {
                    if (!stats) {
                        throw new Error('Provided target path does not exist');
                    }
                    if (stats.isDirectory()) {
                        return targetFlag;
                    }
                    throw new Error('Provided target is not a directory');
                });
            }
            let linkTarget = await this.fuzzyMatchCustomDir(this.args.customerName);
            if (!linkTarget) {
                linkTarget = await this.promptForCustomerDir();
            }
            if (typeof linkTarget === 'string') {
                return linkTarget;
            }
            throw new Error('Could not resolve link target');
        };
        this.getPotentialCustomDirs = (customDir) => {
            if (this.flags.target) {
                return this.flags.target;
            }
            return globby([`${customDir}/**/Custom/`], { onlyDirectories: true })
                .then(dirs => {
                if (dirs.length === 0) {
                    throw new Error(`No subdirectories named "Custom" found in ${customDir}`);
                }
                if (dirs.length === 1) {
                    return dirs[0];
                }
                return prompt([
                    {
                        type: 'list',
                        message: 'Multiple "Custom" dirs found. Which would you like to link?',
                        name: 'dir',
                        choices: dirs.map(dir => {
                            return {
                                value: dir,
                                name: path.relative(path.resolve(customDir, '..'), dir),
                            };
                        })
                    }
                ])
                    .then(({ dir }) => dir);
            });
        };
    }
    async run() {
        const userConfig = await this.requireConfig();
        const linkTarget = await this.getLinkTarget();
        const coreSubdir = await this.getCoreCustomSubdirPath();
        const customSubDir = await this.getPotentialCustomDirs(linkTarget);
        return new Listr([
            {
                title: 'Unlink existing custom dir',
                task: async (_ctx, task) => {
                    task.title = 'Unlinking...';
                    return unlink_1.default.run(this.argv)
                        .then(oldLinkTargetOrFalse => {
                        if (oldLinkTargetOrFalse) {
                            if (typeof oldLinkTargetOrFalse === 'string') {
                                task.title = `Unlinked ${utils_1.decorateRepoPath(path.relative(userConfig.reposDir, oldLinkTargetOrFalse))}`;
                            }
                            else {
                                task.title = 'Unlinked broken symlink';
                            }
                        }
                        else {
                            task.title = 'Nothing to unlink';
                        }
                    });
                }
            },
            {
                title: 'Link',
                task: (_ctx, task) => {
                    return this.makeLink(coreSubdir, customSubDir)
                        .then(() => {
                        const linkedDir = utils_1.decorateRepoPath(path.relative(userConfig.reposDir, coreSubdir));
                        task.output = `Linked "${linkedDir}" => "${customSubDir}"`;
                        task.title = `Linked "${linkedDir}" => "${utils_1.decorateRepoPath(path.relative(userConfig.reposDir, customSubDir))}"`;
                    });
                }
            }
        ]).run();
    }
    makeLink(fromPath, toPath) {
        const linkType = this.config.platform === 'win32' && this.flags['use-junction'] ? 'junction' : 'dir';
        return fs.symlink(toPath, fromPath, linkType);
    }
}
Link.description = 'Link a custom dir to lms-core';
Link.args = [
    {
        name: 'customerName',
        description: 'Partial customer name (fuzzy matched to custom repo names)',
        required: false,
    },
];
Link.examples = [
    chalk_1.default `$ rtolms link     \n\t{gray # prompts for user input to select a customer}`,
    chalk_1.default `$ rtolms link ama \n\t{gray # fuzzy matches customer}`,
    chalk_1.default `$ rtolms link --target=~/Projects/some/dir \n\t{gray # Overrides link target manually}`,
];
Link.flags = Object.assign({}, base_1.default.flags, { 'use-junction': command_1.flags.boolean({
        char: 'j',
        description: 'Use a junction symlink type on windows (defaults to "dir" type)',
    }), target: command_1.flags.string({
        char: 't',
        description: 'Target path to link to',
        parse: input => {
            if (input[0] === '~') {
                return path.resolve(path.join(os.homedir(), input.slice(1)));
            }
            return path.resolve(input);
        },
    }) });
exports.default = Link;
