"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const errors_1 = require("@oclif/errors");
const chalk_1 = require("chalk");
const cheerio = require("cheerio");
const execa = require("execa");
const fs = require("fs-extra");
const globby = require("globby");
const Listr = require("listr");
const path = require("path");
const base_1 = require("../base");
const utils_1 = require("../utils"); //tslint:disable-line no-unused no-unused-variable
class Reconfigure extends base_1.default {
    constructor() {
        super(...arguments);
        this.getConfigurationFiles = async () => {
            const coreRepo = await this.getCorePath();
            const globtions = {
                expandDirectories: true,
                nocase: true,
            };
            const linkedRepoRoot = await this.getLinkedRepoRoot();
            return Promise.all([
                globby(`${coreRepo}/**/connectionStrings.config`, globtions),
                globby(`${coreRepo}/**/app.config`, globtions),
                globby(`${linkedRepoRoot}/**/app.config`, globtions),
                globby(`${coreRepo}/**/smtp.config`, globtions)
            ])
                .then(mdArray => {
                return mdArray.reduce((acc, arr) => {
                    return acc.concat(arr);
                }, []);
            });
        };
        this.getLinkedRepoRoot = async () => {
            const linkedRepo = await this.getCurrentlyLinkedCustomDir();
            if (!linkedRepo) {
                throw new Error('Could not determind currently linked repo root. Is it a git repo?');
            }
            const shell = await execa.shell('git rev-parse --show-toplevel', { cwd: linkedRepo });
            return shell.stdout;
        };
        this.processFile = async (filepath, ctx) => {
            const contents = await fs.readFile(filepath, 'utf8');
            const { server, database, maildrop, } = ctx;
            const stripBom = !this.flags['no-strip-bom'];
            const $ = cheerio.load(contents, {
                xmlMode: true
            });
            $('connectionStrings > [connectionString]').attr('connectionString', (_i, val) => {
                return val.replace(/(Server|Data Source)=.+?;/gi, `$1=${server};`)
                    .replace(/(DataBase|Initial Catalog)=.+?;/gi, `$1=${database};`);
            });
            if (maildrop) {
                $('smtp specifiedPickupDirectory[pickupDirectoryLocation]').attr('pickupDirectoryLocation', maildrop);
            }
            let modifiedContents = $.xml();
            if (stripBom) {
                modifiedContents = modifiedContents.replace(/^&#xFEFF;/, '');
            }
            await fs.writeFile(filepath, modifiedContents);
            return;
        };
    }
    async run() {
        const config = await this.requireConfig();
        let server = this.flags.server;
        let database = this.flags.database;
        if (!server) {
            server = config.server;
        }
        if (!database) {
            database = config.database;
        }
        console.log('');
        if (!server) {
            throw new errors_1.CLIError('No database server configured. Try rerunning `rtolms init` to set one or pass --server');
        }
        if (!database) {
            throw new errors_1.CLIError('No database configured. Try rerunning `rtolms init` to set one or pass --dbName');
        }
        this.log(chalk_1.default `{gray Changing {reset server} => {green ${server}}}\n`);
        this.log(chalk_1.default `{gray Changing {reset database} => {green ${database}}}\n`);
        const filesToMutate = await this.getConfigurationFiles();
        if (filesToMutate.length === 0) {
            throw new errors_1.CLIError('No configuration files found to change with the currently linked customer.');
        }
        return new Listr(filesToMutate.map(filepath => {
            const realpath = path.relative(config.reposDir, fs.realpathSync(filepath));
            return {
                title: utils_1.decorateRepoPath(realpath),
                task: (ctx, _task) => {
                    return this.processFile(filepath, ctx);
                },
            };
        })).run({
            server,
            database,
            maildrop: config.maildrop,
        })
            .then(() => {
            this.log('\nDone.');
        });
    }
}
Reconfigure.description = 'Changes database connection strings';
Reconfigure.aliases = ['reconf', 'reconfig', 'setdb', 'set-db', 'changedb'];
Reconfigure.flags = Object.assign({}, base_1.default.flags, { 'no-strip-bom': command_1.flags.boolean({
        description: 'Do not remove the byte order mark from processed files',
    }), server: command_1.flags.string({
        char: 's',
        description: 'The database server name to change to. Defaults to `config.server`.',
    }), database: command_1.flags.string({
        char: 'd',
        description: 'The name of the database to change to. Defaults to `config.database`.',
    }) });
exports.default = Reconfigure;
