"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const errors_1 = require("@oclif/errors");
const chalk_1 = require("chalk");
const chrono = require("chrono-node");
const globby = require("globby");
const inquirer = require("inquirer");
const Listr = require("listr");
const path = require("path");
const base_1 = require("../../base");
const sql_1 = require("../../sql");
const utils_1 = require("../../utils"); //tslint:disable-line no-unused no-unused-variable
const prompt = inquirer.createPromptModule();
const FROM_EXAMPLES = `
  --from="last 6 months"
  --from="2 months ago"
  --from="June"
  --from="2018-03-01"
  --from="last month"
  --from="jan 1"`;
class SqlInrementals extends base_1.default {
    constructor() {
        super(...arguments);
        this.getPostInstall = async () => {
            const coreDir = await this.getCorePath();
            const coreSqlDir = await this.getSqlDir(coreDir, true);
            const { postInstallScriptFileName } = this.flags;
            const postInstallScript = await globby(`${coreSqlDir}/${postInstallScriptFileName}`);
            if (!postInstallScript.length) {
                throw new Error('No postInstallScript found');
            }
            return postInstallScript[0];
        };
        this.getSqlScripts = async (baseDir) => {
            const sqlDir = await this.getSqlDir(baseDir, this.flags.core);
            const fromDate = chrono.parseDate(this.flags.from);
            const allScripts = this.flags.archived ? await globby(sqlDir + '/**/*.sql') : await globby(sqlDir + '/*.sql');
            const sortedScripts = await Promise.all(allScripts.map(async (filepath) => {
                return {
                    filepath,
                    date: await utils_1.getFileDateByFilenameOrGitAuthorDate(filepath)
                        .catch(err => {
                        this.error(err);
                        return new Date();
                    }),
                };
            }))
                .then(scripts => {
                return scripts
                    .sort((a, b) => {
                    return a.filepath > b.filepath ? -1 : a.filepath < b.filepath ? 1 : 0;
                })
                    .sort((a, b) => {
                    return a.date > b.date ? -1 : a.date < b.date ? 1 : 0;
                });
            });
            return prompt([
                {
                    type: 'checkbox',
                    message: 'Which scripts would you like to run?',
                    name: 'scriptsToRun',
                    choices: [
                        ...sortedScripts.map((file) => {
                            return {
                                value: file.filepath,
                                name: path.relative(baseDir, file.filepath),
                                checked: file.date > fromDate,
                            };
                        }),
                        {
                            type: 'separator',
                        }
                    ],
                }
            ])
                .then(({ scriptsToRun }) => scriptsToRun.reverse());
        };
        this.getSqlDir = async (baseDir, core) => {
            const sqlDirs = await globby([
                core ? 'Sql/Incremental' : 'Sql/',
                '*/Sql',
            ], {
                cwd: baseDir,
                absolute: true,
                onlyDirectories: true,
                matchBase: true,
            });
            if (sqlDirs.length === 0) {
                return this.error('No Sql subdirectories found');
            }
            if (sqlDirs.length > 1) {
                return prompt([
                    {
                        type: 'list',
                        message: 'Multiple "Sql" dirs found. Which would you like to run scripts from?',
                        name: 'dir',
                        choices: sqlDirs.map(dir => {
                            return {
                                value: dir,
                                name: dir,
                            };
                        })
                    }
                ])
                    .then(({ dir }) => dir);
            }
            return sqlDirs[0];
        };
        this.getCustomDirOrPrompt = async () => {
            const useLinked = !this.flags['no-use-linked'];
            let linkedDir;
            if (useLinked) {
                linkedDir = await this.getCurrentlyLinkedCustomDir();
                if (!linkedDir) {
                    this.warn('No currently linked custom directory found.');
                }
            }
            if (!linkedDir) {
                linkedDir = await this.fuzzyMatchCustomDir(this.args.customerName);
            }
            if (!linkedDir) {
                this.log('Which would you like to run incrementals for?');
                linkedDir = await this.promptForCustomerDir();
            }
            if (path.basename(linkedDir).toLowerCase() === 'custom') {
                linkedDir = path.resolve(linkedDir, '..');
            }
            if (!linkedDir) {
                throw new Error('Could not resolve link target');
            }
            return linkedDir;
        };
    }
    async run() {
        const userConfig = await this.requireConfig();
        let corePath = await this.getCorePath();
        let repoDir;
        const { core, username, password, verbose, } = this.flags;
        if (!chrono.parseDate(this.flags.from)) {
            throw new errors_1.CLIError(`Could not parse "${this.flags.from}" as a date.\n\nTry using one of these formats:${FROM_EXAMPLES}`);
        }
        if (core) {
            repoDir = corePath;
        }
        else {
            repoDir = await this.getCustomDirOrPrompt();
        }
        const scripts = await this.getSqlScripts(repoDir);
        const postInstallScript = await this.getPostInstall();
        scripts.push(postInstallScript);
        const listrOpts = {
            renderer: (() => {
                if (!process.stdout.isTTY)
                    return 'verbose';
                return verbose ? 'verbose' : 'default';
            })(),
            exitOnError: true,
        };
        const [cfgServer, cfgInstanceName,] = (this.flags.server || userConfig.server).split('\\');
        const server = cfgServer.replace(/^\.$/, 'localhost');
        const connectionConfig = {
            user: username,
            password,
            server,
            database: this.flags.database || userConfig.database,
            instanceName: this.flags.instance || cfgInstanceName,
            connectionTimeout: 5000,
            requestTimeout: 5000,
            pool: {
                max: 1,
                min: 0,
            },
            options: {
                encrypt: false,
            }
        };
        this.conn = await sql_1.createConnection(connectionConfig)
            .catch(err => {
            this.error(err);
        });
        return new Listr(scripts.map(filepath => {
            return {
                title: utils_1.decorateRepoPath(path.relative(userConfig.reposDir, filepath)),
                task: (_ctx, task) => {
                    return sql_1.executeSqlFile({
                        filepath,
                    }, this.conn)
                        .then(resultSet => {
                        task.output = `Rows affected: ${utils_1.inspect(resultSet.rowsAffected)}`;
                        return resultSet;
                    });
                }
            };
        }), listrOpts).run();
    }
    finally() {
        if (this.conn) {
            this.conn.close();
        }
        return Promise.resolve();
    }
}
SqlInrementals.description = chalk_1.default `Run SQL incremental scripts

This command will look for {cyan .sql} scripts in subdirectories of the selected repo
named "{cyan Sql}". If any are found, it will prompt you to select which scripts to
run before executing them.

The {yellow --from} flag can be used to preselect scripts which are dated after
the given time. See examples for more.
`;
SqlInrementals.aliases = ['sql', 'sql:index'];
SqlInrementals.args = [
    {
        name: 'customerName',
        description: 'Partial customer name (fuzzy matched to custom repo names)',
        required: false,
    },
];
SqlInrementals.examples = [
    chalk_1.default `$ rtolms sql:incrementals     \n\t{gray # finds scripts in your currently linked custom dir}`,
    chalk_1.default `$ rtolms sql:incrementals -L  \n\t{gray # prompts for a custom repo to look for scripts in}`,
    chalk_1.default `$ rtolms sql:incrementals --core --from="jul 4"     \n\t{gray # finds scripts in your lms-core dir and preselects all dated after July 4 of this year}`,
    chalk_1.default `$ rtolms sql:incrementals ama \n\t{gray # finds scripts in fuzzy matched lms-custom-aama dir}`,
    chalk_1.default `$ rtolms sql:incrementals -s="my-vm.local" \n\t{gray # connects to server on "my-vm.local" (port 1433)}`,
    chalk_1.default `$ rtolms sql:incrementals -s="my-vm.local\\\\SQLEXPRESS" \n\t{gray # connects to server instance "SQLEXPRESS" on "my-vm.local"}`,
    chalk_1.default `$ rtolms sql:incrementals -ca \n\t{gray # runs archived incrementals from core}`,
];
SqlInrementals.flags = Object.assign({}, base_1.default.flags, { verbose: command_1.flags.boolean({ char: 'v', description: 'Verbose' }), 'no-use-linked': command_1.flags.boolean({
        char: 'L',
        description: 'Ignore currently linked custom dir',
    }), core: command_1.flags.boolean({
        char: 'c',
        description: 'Run core incrementals instead of custom',
        exclusive: ['no-use-linked'],
    }), archived: command_1.flags.boolean({
        char: 'a',
        description: 'Show archived incrementals also',
    }), from: command_1.flags.string({
        description: `Preselect scripts dated after FROM. See\n[parser](https://github.com/wanasit/chrono) for more info on matching.\n${FROM_EXAMPLES}\n`,
        default: 'last 6 months',
        required: true,
    }), server: command_1.flags.string({
        char: 's',
        description: 'The database server name to change to. Defaults to `config.server`.',
    }), database: command_1.flags.string({
        char: 'd',
        description: 'The name of the database to change to. Defaults to `config.database`.',
    }), instance: command_1.flags.string({
        description: 'The sql server instance on the server (Must have port 1434 open). Defaults to server.split(\\)[1]',
    }), username: command_1.flags.string({
        char: 'u',
        description: 'The username to connect to the db with.',
        default: 'dev',
    }), password: command_1.flags.string({
        char: 'p',
        description: 'The password to connect to the db with.',
        required: true,
        default: 'elopment',
    }), postInstallScriptFileName: command_1.flags.string({
        hidden: true,
        description: 'The name of the postInstallScript file to be run after all incrementals',
        default: '90000000-PostInstall.sql',
    }) });
exports.default = SqlInrementals;
