"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
var dumper_js_1 = require("dumper.js");
exports.dd = dumper_js_1.dd;
const chrono = require("chrono-node");
const execa = require("execa");
const fs = require("fs-extra");
const hyperlinker = require("hyperlinker");
const inquirer = require("inquirer");
const path = require("path");
const supportsHyperlinks = require("supports-hyperlinks");
const util = require("util");
exports.inspect = (obj) => util.inspect(obj, { colors: true, depth: 3 });
const decorateRepoPathDefaults = {
    reset: true,
    hyperlink: true,
    basenameOnly: false,
};
function decorateRepoPath(repoPath, options) {
    const { reset, hyperlink, basenameOnly } = Object.assign({}, decorateRepoPathDefaults, options);
    const doHyperlink = hyperlink && supportsHyperlinks.stdout && path.isAbsolute(repoPath);
    let ret = repoPath;
    if (basenameOnly) {
        ret = path.basename(ret);
    }
    ret = ret
        .replace(/(lms-)(core)(\/|$)/g, chalk_1.default `${reset ? chalk_1.default.reset('$1') : '$1'}{cyan $2}$3`)
        .replace(/(\/)(Custom)/g, chalk_1.default `$1{bold $2}`)
        .replace(/(lms-custom-)(\w+)/g, chalk_1.default `${reset ? chalk_1.default.reset('$1') : '$1'}{cyan $2}`)
        .replace(/(lms-core-legacy-)(\w+)/g, chalk_1.default `${reset ? chalk_1.default.reset('$1') : '$1'}{cyan $2}`);
    if (doHyperlink) {
        ret = hyperlinker(ret, `file://${repoPath}`);
    }
    return ret;
}
exports.decorateRepoPath = decorateRepoPath;
async function promptForConfig() {
    const prompt = inquirer.createPromptModule();
    const flags = this.flags;
    const cfgPath = `${this.config.configDir}/config.json`;
    const defaultConfigValues = await this.getConfigDefaults();
    const existingCfg = await (() => {
        return fs.readJSON(cfgPath)
            .catch(() => {
            return {};
        });
    })();
    if (flags.list) {
        console.log(exports.inspect(existingCfg));
        process.exit();
    }
    const defaults = Object.assign({}, defaultConfigValues, existingCfg);
    return prompt([
        {
            type: 'input',
            message: 'What is your GitHub username?',
            default: defaults.username || path.basename(this.config.home),
            name: 'username',
        },
        {
            type: 'input',
            message: 'What is your GitHub auth token?',
            name: 'authToken',
            validate: val => val.length < 2 ? 'Token must be 41 characters' : true,
            default: defaults.authToken,
        },
        {
            type: 'input',
            message: 'Where do you want to store your repos?',
            name: 'reposDir',
            filter: userInp => {
                let val = userInp;
                if (val.charAt(0) === '~') {
                    val = val.replace('~', this.config.home);
                }
                return path.resolve(process.cwd(), path.normalize(val));
            },
            default: defaults.reposDir,
        },
        {
            type: 'input',
            message: 'Database server name',
            name: 'server',
            default: defaults.server,
        },
        {
            type: 'input',
            message: 'Database name',
            name: 'database',
            default: defaults.database,
        },
        {
            type: 'input',
            message: 'Maildrop location',
            name: 'maildrop',
            default: defaults.maildrop,
        },
    ])
        .then((answers) => {
        console.log('');
        console.log(exports.inspect(answers));
        console.log(`\nThis configuration will be written to:\n\t${cfgPath}`);
        return prompt([
            {
                type: 'confirm',
                message: 'Look good?',
                default: true,
                name: 'confirmed',
            },
        ])
            .then(({ confirmed }) => {
            if (!confirmed) {
                this.log('KBAI.');
                process.exit(0);
            }
            return answers;
        });
    })
        .then((answers) => {
        return fs.ensureDir(this.config.configDir)
            .then(() => {
            return fs.writeFile(cfgPath, JSON.stringify(answers, undefined, '  '))
                .then(() => {
                return answers;
            });
        });
    });
}
exports.promptForConfig = promptForConfig;
async function getFileDateByFilenameOrGitAuthorDate(filepath) {
    let filedate;
    try {
        const fn = path.basename(filepath);
        const match = fn.match(/(\d{4})-?(\d{2})-?(\d{2})/);
        if (match) {
            if (match[1] === '9000') {
                filedate = new Date(9000, 0);
            }
            else {
                filedate = chrono.parseDate(match.slice(1, 4).join('-'));
            }
        }
    }
    catch (_a) {
        //pass
    }
    if (!filedate) {
        try {
            const gitDateOut = await execa.shell(`git -C "${path.dirname(filepath)}" log --format=%aI ${filepath}`);
            const gitDate = gitDateOut.stdout.split('\n').reverse()[0];
            if (gitDate) {
                filedate = new Date(gitDate);
            }
        }
        catch (_b) {
            // console.log('Could not get file date from git')
        }
    }
    if (!filedate) {
        // ¯\_(ツ)_/¯
        throw new Error(`Could not get date for file "${filepath}"`);
        filedate = new Date();
    }
    return filedate;
}
exports.getFileDateByFilenameOrGitAuthorDate = getFileDateByFilenameOrGitAuthorDate;
